%% estimate the VARMA(p,q)-SV1

% initialize the chain 
bigX = SURform2(X,n);
Ttn = Tt*n;
beta = (bigX'*bigX)\(bigX'*Yt);
err = reshape(Yt - bigX*beta,n,Tt);
Lam = diag(err*err')/Tt/2;
h = repmat(log(Lam)',Tt,1);
B = zeros(n,(q+1)*n);
B(:,1:n) = eye(n);
Phi = buildPhi(B,Tt);
Sigh = .01*ones(n,1);    
    
tempyhat1 = zeros(nloop-burnin,n+1); 
tempyhat2 = zeros(nloop-burnin,n+1); 
tempyhat3 = zeros(nloop-burnin,n+1);

rand('state', sum(100*clock) ); randn('state', sum(200*clock) );
    
for loop = 1:nloop
    
        %% sample beta (marginal of f)
    Sy = sparse(1:Ttn,1:Ttn,repmat(Lam,Tt,1)) ...
        + Phi*sparse(1:Ttn,1:Ttn,reshape(exp(h)',Ttn,1))*Phi';      
    invDbeta = invVbeta + bigX'*(Sy\bigX);
    betahat = invDbeta\(bigX'*(Sy\Yt));
    Cbeta = chol(invDbeta,'lower');
    flag = 0;
    while flag == 0  %% impose stationarity
        beta =  betahat + Cbeta'\randn(kb,1);
        A = reshape(beta,p*n+1,n)';
        P(1:n,:) = A(:,2:end);
        if sum(abs(eig(P))<1) == p*n
            flag = 1;
        end
    end
    Xbeta = bigX*beta;
    
        %% sample f
    PhiinvSigLam = Phi'*sparse(1:Ttn,1:Ttn,repmat(1./Lam,Tt,1));
    invDf = sparse(1:Ttn,1:Ttn,reshape(exp(-h)',Ttn,1)) + PhiinvSigLam*Phi;
    fhat = invDf\(PhiinvSigLam*(Yt-Xbeta));
    f = fhat + chol(invDf,'lower')'\randn(Ttn,1);
    shortf = reshape(f,n,Tt)';
    
        %% sample h  
    for i=1:n
        Ystar = log(shortf(:,i).^2 + .0001 );
        h(:,i) = SVRW(Ystar,h(:,i),Sigh(i),Vh);
    end
    
        %% sample Phi    
    Ystar = reshape(Yt-Xbeta,n,Tt)';
    for i=1:n
        kj = i-1 + n*q; 
        Z = zeros(Tt,kj); 
        Z(:,1:i-1) = shortf(:,1:i-1);
        for ii = 1:q
            Z(:,i+(ii-1)*n:i+ii*n-1) = [zeros(ii,n); shortf(1:end-ii,:)];
        end
        invDphi = sparse(1:kj,1:kj,[ones(1,i-1)*invVphi0 ones(1,n*q)*invVphi1]) ...
            + Z'*Z/Lam(i);
        phihat = invDphi\(Z'*(Ystar(:,i)-shortf(:,i))/Lam(i));
        phi = phihat + chol(invDphi,'lower')'\randn(kj,1);
        B(i,[1:i-1 n+1:n*(q+1)]) = phi;
    end
    Phi = buildPhi(B,Tt);
    
        %% sample Lam
    erry = reshape(Yt - Xbeta - Phi*f,n,Tt)';
    newSLam = S0Lam + sum(erry.^2)'/2;
    Lam = 1./gamrnd(nuLam + Tt/2,1./newSLam);
    
        %% sample Sigh
    e2 = (h(2:Tt,:) - h(1:Tt-1,:)).^2;
    Sigh = sample_Sig(Sigh,e2,VSigh);
    
    if loop>burnin %% compute various forecasts
       compute_forecast_VARMA_SV1; 
    end    
end     
       
maxy1 = max(tempyhat1);
maxy2 = max(tempyhat2);
maxy3 = max(tempyhat3);    
